# (1.2) Using functions in the data module, create data and header variables
# for baseball.csv, census.csv, and bluebikes.csv
import data
from statistics import mean
from collections import Counter

baseball_data = data.data_as_list_of_lists("baseball.csv")
baseball_header = data.data_header("baseball.csv")

bluebikes_data = data.data_as_list_of_lists("bluebikes.csv")
bluebikes_header = data.data_header("bluebikes.csv")

census_data = data.data_as_list_of_lists("census.csv")
census_header = data.data_header("census.csv")

# (1.3) Using the head() function from data.py, print the first 5 rows (and headers) of each
# dataset.

data.head(baseball_header, baseball_data, 5)
data.head(bluebikes_header, bluebikes_data, 5)
data.head(census_header, census_data, 5)

# (2.1) Use a for loop to create a list of all the distances in the bluebikes dataset,
# then use the max, min, and mean functions to calculate the minimum, maximum, and
# mean distances in the dataset.
distances = []
for row in bluebikes_data:
    distances.append(row[7])
print(len(distances))
print(len(bluebikes_data))
print(distances[0:5])

# (2.2) Instead of a for loop, use a list comprehension to create the list of
# distances. Confirm that the output is the same as in 2.1.
distances_lc = [row[7] for row in bluebikes_data]
print(len(distances_lc))
print(distances_lc[0:5])

# (3.1). Write a function to generalize getting a column from a dataset in list form.


def get_column(dataset, idx):
    """
    For a dataset in list-of-lists form, return a list containing the indexed value of
    each row.

    Parameters:
        dataset (list of lists): the input data
        idx (int): an integer index corresponding to a column in the data

    Returns:
        column (list): a single list containing a value from each row
    """
    return [row[idx] for row in dataset]


distances_fun = get_column(bluebikes_data, 7)
print(len(distances_fun))
print(distances_fun[0:5])


# (3.2) Write a function that takes a column (list) as input, and returns True
# if it contains numeric data and False otherwise. For example, the output
# should look like this:
# > bb_fnames = get_column(baseball_data, 0)
# > bb_years = get_column(baseball_data, 2)
# > print(is_numeric(bb_fnames))
# False
# > print(is_numeric(bb_years))
# True


def is_numeric(column):
    """
    For a dataset and a column index, return True if the column is numeric
     and False if it is not.

    HINT: we can assume that each column is of one type, i.e., we can consider
    only the first element of the column.

    Parameters:
        column (list): a list of values corresponding to a column in a dataset

    Returns:
        answer (bool): True if the column is numeric, False otherwise
    """
    if type(column[0]) == int or type(column[0]) == float:
        return True
    else:
        return False
    # the following is a "nicer" version
    # if isinstance(column[0], (int, float)):
    #     return True
    # else:
    #     return False


bb_fnames = get_column(baseball_data, 0)
bb_years = get_column(baseball_data, 2)
print(is_numeric(bb_fnames))
print(is_numeric(bb_years))

# (3.3) Write a function that takes a dataset and its associated header, and
# prints out the names of the numeric columns. For example, the output should
# look like this:
# > print_columns(census_header)
# 0 Year
# 1 Subunit
# 2 FName
# 3 Surname
# 4 Age
# 5 Sex
# 6 Color
# 7 Profession

def print_columns(header):
    """
    Take a header and print the names *and* indices of each column.

    Hint: Use range(len(header))

    Parameters:
        header (list of strings): the names of each column in the dataset

    Returns:
        None
    """
    for col_idx in range(len(header)):
        print(col_idx, header[col_idx])
    return None


print_columns(bluebikes_header)

# (3.4) Write a function that takes a dataset and a header and prints the
# name, maximum, minimum, and mean.
# Hint: Use is_numeric and get_column. The logic is quite close to that of
# print_numeric_columns, but you won't want to use that function directly in
# summarize().


def summarize(dataset, header):
    """
    For a dataset, take each numeric column and print its name, minimum,
    maximum, and mean.

    Parameters:
        dataset (list of lists): the input data
        header (list of strings): the names of each column in the dataset

    Returns:
        None
    """
    for col_idx in range(len(header)):
        col = get_column(dataset, col_idx)
        if is_numeric(col):
            print(header[col_idx])
            print("min:", round(min(col), 2))
            print("mean:", round(mean(col), 2))
            print("max: ", round(max(col), 2))
            print()
    return None


summarize(bluebikes_data, bluebikes_header)
summarize(census_data, census_header)
summarize(baseball_data, baseball_header)

# (4) Rewrite the filtering functions in data.py using list comprehensions.


def filter_numerical(dataset, col_index, min_value, max_value):
    """
    Filters the dataset by a numerical column and return the specified subset

    Parameters:
        dataset (list of lists): the input data
        col_index (int): an integer index corresponding to a column in the data
        max/min_values (float): the minimum and maximum values to filter on

    Returns:
        filtered_data (list of lists): the filtered data, with the same column structure as dataset
    """
    return [row for row in dataset if row[col_index] >= min_value and row[col_index] <= max_value]


def filter_categorical(dataset, col_index, categories):
    """
    Filters the dataset by a categorical column and return the specified subset

    Parameters:
        dataset (list of lists): the input data
        col_index (int): an integer index corresponding to a column in the data
        categories (list): a list of strings representing categories to include

    Returns:
        filtered_data (list of lists): the filtered data, with the same column structure as dataset
    """
    return [row for row in dataset if row[col_index] in categories]


print(filter_categorical(baseball_data, 4, ["MON"]) == data.filter_categorical(baseball_data, 4, ["MON"]))
print(filter_numerical(bluebikes_data, 0, 5561, 5561) == data.filter_numerical(bluebikes_data, 0, 5561, 5561))

def summarize(dataset, header):
    """
    For a dataset, take each numeric column and print its name, minimum,
    maximum, and mean.

    Parameters:
        dataset (list of lists): the input data
        header (list of strings): the names of each column in the dataset

    Returns:
        None
    """
    for col_idx in range(len(header)):
        col = get_column(dataset, col_idx)
        if is_numeric(col):
            print(header[col_idx])
            print("min:", round(min(col), 2))
            print("mean:", round(mean(col), 2))
            print("max: ", round(max(col), 2))
            print()
        else:
            print(header[col_idx])
            top3 = Counter(col).most_common(3)
            top3names = [x[0] for x in top3]
            print("\n".join(top3names))
            print()

    return None


# summarize(bluebikes_data, bluebikes_header)
# summarize(census_data, census_header)
# summarize(baseball_data, baseball_header)
